#! /usr/bin/python3
#-*- encoding: utf-8 -*-
import sys
from os.path import isdir, isfile, join
from os import makedirs, listdir, rename, unlink
from shutil import rmtree
from zipfile import ZipFile
from json import dumps
import traceback
from pyeole.lock import acquire, release
from argparse import ArgumentParser

from eoleaaf.parseaaf import main as complet_main
from eoleaaf.parsedelta import main as delta_main
from eoleaaf.config import path_aaf_complet
from eoleaaf.util import db
from eoleaaf.aaf import reset_db
from logging import getLogger

aaflog = getLogger("AAF")

def processfile(filename,
                level,
                delete=False,
                erase_db=False,
                disable_transaction=False,
                ):
    """
    :param filename: fullpath of the archive to unzip
    :param level: type of import ('complète' or 'delta')
    :param delete: if True, remove the import file after successfull process
    :param erase_db: if True, remove all element in database
    :param disable_transaction: do not allow bulk import
    """
    aaflog.info(20*'*')
    aaflog.info('*** Début du traitement des fichiers AAF (importation {1}) : {0}'.format(filename, level))
    if level == 'complète':
        main = complet_main
    elif level == 'delta':
        main = delta_main
    else:
        aaflog.error('level inconnu {}'.format(level))
        sys.exit(1)

    # the file must exists
    if not isfile(filename):
        aaflog.error("le fichier {} n'existe pas".format(filename))
        sys.exit(3)
    try:
        acquire('importaaf', level="system")
    except Exception as err:
        aaflog.error('impossible de placer le fichier lock {}'.format(err))
        sys.exit(2)

    if erase_db:
        reset_db()
    try:
        if db.db.user.find_one():
            maj_mode = level == 'complète'
        else:
            if level == 'delta':
                release('importaaf', level='system')
                sys.exit(4)
            maj_mode = level != 'complète'
    except Exception as err:
        traceback.print_exc()
        release('importaaf', level='system')
        sys.exit(5)

    # purge the folders
    try:
        if isdir(path_aaf_complet):
            rmtree(path_aaf_complet)
        makedirs(path_aaf_complet)
    except Exception as err:
        traceback.print_exc()
        release('importaaf', level='system')
        sys.exit(6)

    # unzip the files
    try:
        zip_ref = ZipFile(filename, 'r')
        zip_ref.extractall(path_aaf_complet)
        zip_ref.close()
        # move the files in path_aaf_complet if there is any subfolder
        for fname in listdir(path_aaf_complet):
            fullname = join(path_aaf_complet, fname)
            if isdir(fullname):
                for subfname in listdir(fullname):
                    rename(join(fullname, subfname), join(path_aaf_complet, subfname))
    except Exception as err:
        traceback.print_exc()
        release('importaaf', level='system')
        sys.exit(7)

    try:
        # imports the files in mongodb
        main(path_aaf_complet,
             maj_mode=maj_mode,
             allow_bulk=not disable_transaction,
             )
    except Exception as err:
        traceback.print_exc()
        release('importaaf', level='system')
        sys.exit(8)

    # Delete AAF zip file if success
    if delete:
        unlink(filename)
        aaflog.info('Suppression du fichier {0}'.format(filename))

    aaflog.info('*** Fin du traitement des fichiers AAF')
    aaflog.info(20*'*')
    release('importaaf', level='system')
    sys.exit(0)

if __name__ == "__main__":
    parser = ArgumentParser(description="Importation des fichiers AAF dans la base de donnée MongoDB")
    parser.add_argument("filename", help="nom du fichier ZIP contenant les fichiers AAF")
    parser.add_argument("level",
                        choices=['complète', 'delta'],
                        help="niveau de fichier AAF")
    parser.add_argument("--delete",
                        help="supprime le fichier AAF en fin d'import",
                        action="store_true")
    parser.add_argument("--disable_transaction",
                        help="désactiver les transactions",
                        action="store_true")
    parser.add_argument("--erase_db",
                        help="commence par supprimer le contenu de la base de donnée (a n'utiliser que si vous savez ce que vous faites !)",
                        action="store_true")
    args = parser.parse_args()
    processfile(args.filename,
                args.level,
                args.delete,
                args.erase_db,
                args.disable_transaction,
                )
